/**
 * LogFile plugin
 *
 * by Stephan Slabihoud
 *
 * 0.0x: Testversions
 * 1.00: official Release
 * 1.10: Evaluates xSpamPalReason
 * 1.11: New: Open logfile
 * 1.2:  New: Additional data logged into logfile.log
 * 1.3:  Tabs will be removed
 * 1.4:  Servername added
 * 1.5:  Check if started from SpamPal
 * 1.6:  Buffer overflow fixed
 * 		 GetLocalTime
 * 1.7:  SURBL support added
 **/

#include <windows.h>
#include <shellapi.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include "spampal.h"
#include "logfile.h"
#include "mytypes.h"

#define PLUGINVERSION "1.7"
#define PLUGINNAME    "Logfile"

HINSTANCE dllInstance;

CRITICAL_SECTION critSect;
int numProcs = 0; // number of processes using this DLL
                  // when ==0, critical section is destroyed.


char	logdir[MAX_STRING];

BOOL WINAPI LibMain(HINSTANCE hDLLInst, DWORD fdwReason, LPVOID lpvReserved)
{
    switch (fdwReason)
    {
        case DLL_PROCESS_ATTACH:	dllInstance = hDLLInst;
									if (numProcs==0) {
										DisableThreadLibraryCalls((HMODULE)hDLLInst);
										InitializeCriticalSection(&critSect);
									}
									numProcs++;
						            // The DLL is being loaded for the first time by a given process.
						            // Perform per-process initialization here.  If the initialization
            						// is successful, return TRUE; if unsuccessful, return FALSE.

						            break;
        case DLL_PROCESS_DETACH:    // The DLL is being unloaded by a given process.  Do any
            						// per-process clean up here, such as undoing what was done in
            						// DLL_PROCESS_ATTACH.  The return value is ignored.
									numProcs--;
									if (numProcs==0)
										DeleteCriticalSection(&critSect);
            						break;
        case DLL_THREAD_ATTACH:		// A thread is being created in a process that has already loaded
            						// this DLL.  Perform any per-thread initialization here.  The
            						// return value is ignored.
						            break;
        case DLL_THREAD_DETACH:     // A thread is exiting cleanly in a process that has already
            						// loaded this DLL.  Perform any per-thread clean up here.  The
            						// return value is ignored.
            						break;
    }
    return TRUE;
}



const char* getPluginName() {
	return PLUGINNAME;
}


const char* getPluginVersion() {
	return PLUGINVERSION;
}


DWORD loadConfig(const char* dir) {
	EnterCriticalSection(&critSect);
	strcpy(logdir, dir);
	LeaveCriticalSection(&critSect);
	return 1;
}

void releaseConfig(DWORD handle) {
}


BOOL CALLBACK infoDialogProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam) {
	POINTS pts;
	POINT pt;
	HWND ctl;
	HCURSOR pointingHandCursor;

	switch (msg) {

        case WM_CTLCOLORSTATIC:
			if ((HWND) lParam == GetDlgItem(hwnd, LBL_URL)) {
            	SetTextColor((HDC) wParam, RGB(0, 0, 255));
				SetBkColor((HDC) wParam, GetSysColor(COLOR_3DFACE));
				return (BOOL)GetSysColorBrush(COLOR_3DFACE);
			} else
				return FALSE;
			break;

		case WM_MOUSEMOVE:  // I tried using WM_SETCURSOR for this but couldn't make it work.
		    pointingHandCursor = LoadCursor(dllInstance,MAKEINTRESOURCE(IDCURSORHAND));
			pts = MAKEPOINTS(lParam);
			pt.x = pts.x;
			pt.y = pts.y;
			ctl = ChildWindowFromPoint(hwnd, pt);
			if (ctl==GetDlgItem(hwnd, LBL_URL)) {
				SetCursor(pointingHandCursor);
			}
			break;

		case WM_LBUTTONUP:
			pts = MAKEPOINTS(lParam);
			pt.x = pts.x;
			pt.y = pts.y;
			ctl = ChildWindowFromPoint(hwnd, pt);
			if (ctl == GetDlgItem(hwnd, LBL_URL)) {
				ShellExecute(hwnd, "open", "http://www.slabihoud.de/spampal/", NULL, NULL, SW_SHOWMAXIMIZED);
			}
			else {
				return FALSE;
			}
		case WM_CLOSE:
			ShowWindow(hwnd, SW_HIDE);
			return TRUE;
		case WM_COMMAND:
			switch (LOWORD(wParam)) {
				case CMD_INFO_DISMISS:
					ShowWindow(hwnd, SW_HIDE);
					return TRUE;
			}
	}
	return FALSE;
}


HWND createInfoDialog(HWND parent) {
	return CreateDialog(dllInstance, MAKEINTRESOURCE(DLG_ABOUT), parent, infoDialogProc);
}


BOOL CALLBACK configDialogProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam) {
	HWND lst;
	switch (msg) {
		case WM_CLOSE:
			ShowWindow(hwnd, SW_HIDE);
			return TRUE;
		case WM_SHOWWINDOW:
			return FALSE;
		case WM_COMMAND:
			switch (LOWORD(wParam)) {
				case CMD_CONFIG_DISMISS:
					ShowWindow(hwnd, SW_HIDE);
					return TRUE;
				case CMD_CONFIG_OPENLOG:
					ShellExecute(hwnd, "open", logdir, NULL, NULL, SW_SHOWNORMAL);
					return TRUE;
			}
	}
	return FALSE;
}


HWND createConfigDialog(DWORD configHandle, HWND parent) {
	HWND dlg;
	dlg = CreateDialog(dllInstance, MAKEINTRESOURCE(DLG_CONFIG), parent, configDialogProc);
	return dlg;
}

BOOL validateConfigDialog(HWND dlg, DWORD oldConfigHandle) {
	return TRUE;
}

void saveConfigFromDialog(HWND dlg, DWORD oldConfigHandle, const char* dir) {
}

void filterOnMessageHeaders(
	const char** headerLines,
	const int num_header_lines,
	unsigned long fetchHandle,
	struct MessageStatus* msgStatus,
	const DWORD configHandle,
	const struct SpamPalFilteringHooks* spampalHooks
) {
}

BOOL requireBodyFiltering(
	const char** headerLines,
	int num_header_lines,
	unsigned long fetchHandle,
	const struct MessageStatus* msgStatus,
	DWORD configHandle
) {
	return FALSE;
}

void filterOnMessageBody(
	const char** headerLines,
	const int num_header_lines,
	const HANDLE bodyHandle,
	const unsigned long fetchHandle,
	struct MessageStatus* msgStatus,
	const DWORD configHandle,
	const struct SpamPalFilteringHooks* spampalHooks
) {
}

/*******************************************************************/
/*********************** processedMessage **************************/
/*******************************************************************/

char *getDateTimeID(void) {
long sec_now;
	struct tm *tm_now;
	static char s[128],temp[128];

 	SYSTEMTIME now;
	int year;
	GetLocalTime(&now);
	year = now.wYear-1900;

	time(&sec_now);
	tm_now=localtime(&sec_now);
	tm_now->tm_year %= 100;											/* Y2K */

	strftime(temp,30,"%y%m%d-",tm_now);
	strcpy(s,temp);
	sprintf(temp, "%02d%02d%02d",now.wHour,now.wMinute,now.wSecond);
	strcat(s,temp);
	return &s[0];
}

char *getFolderDateID(void) {
	static char s[128];
	long sec_now;
	struct tm *tm_now;
	time(&sec_now);
	tm_now=localtime(&sec_now);
	strftime(s,30,"%Y-%W",tm_now);
	return &s[0];
}

void convertTabs(char *str) {
	while(*str) {
		if (*str==8 || *str==10 || *str==13)
			*str=' ';
		str++;
	}
}

void trimSpaces(char* str)
{
	char *pointer;
	char *old=str;
	while (isspace(*str) && (*str)) str++;
	pointer = strchr(str,EOS);
	while (isspace(*(--pointer)) && pointer>str) *pointer = EOS;
	strcpy(old,str);
}

CRITICAL_SECTION critSect;
#define BODY_BUF_SIZE 4096

void processedMessage(
	const char** headerLines,
	int num_header_lines,
	HANDLE bodyHandle,
	unsigned long fetchHandle,
	const struct MessageStatus* msgStatus,
	DWORD configHandle
) {
	DWORD bytesRead;
	char type  [BODY_BUF_SIZE+1];
	char name  [BODY_BUF_SIZE+1];
	char su_buf[BODY_BUF_SIZE+1];
	char fr_buf[BODY_BUF_SIZE+1];
	char to_buf[BODY_BUF_SIZE+1];
	char the_ip[BODY_BUF_SIZE+1];
	char dnsbl [BODY_BUF_SIZE+1];
	char spam_r[BODY_BUF_SIZE+1];
	BOOL err;
	char filelog[MAX_STRING];
	char filelog2[MAX_STRING];

	if (!msgStatus->msgInfo->isComplete)
 		return;

	EnterCriticalSection(&critSect);


	sprintf(filelog,  "%s%s.txt", logdir, getFolderDateID());
	sprintf(filelog2, "%s%s.txt", logdir, "logfile");


	FILE *fh  = fopen(filelog,  "ab");
	FILE *fh2 = fopen(filelog2, "ab");

	if (fh!=NULL)  fprintf(fh, "%s", getDateTimeID());
	if (fh2!=NULL) fprintf(fh2, "%s", getDateTimeID());

	*su_buf =
	*fr_buf =
	*to_buf =
	*the_ip =
	*dnsbl  =
	*spam_r =
	*name   = EOS;
	for (int i=0;i<num_header_lines;i++) {
		if (headerLines[i]) {
			if (!strnicmp(headerLines[i],"Subject: ",9)) {
				strncpy(su_buf,headerLines[i]+9,BODY_BUF_SIZE);
				su_buf[BODY_BUF_SIZE] = EOS;
			}
			if (!strnicmp(headerLines[i],"From: ",6)) {
				strncpy(fr_buf,headerLines[i]+6,BODY_BUF_SIZE);
				fr_buf[BODY_BUF_SIZE] = EOS;
			}
			if (!strnicmp(headerLines[i],"To: ",4)) {
				strncpy(to_buf,headerLines[i]+4,BODY_BUF_SIZE);
				to_buf[BODY_BUF_SIZE] = EOS;
			}
		}
	}
	trimSpaces(su_buf);
	trimSpaces(fr_buf);
	trimSpaces(to_buf);
	convertTabs(su_buf);
	convertTabs(fr_buf);
	convertTabs(to_buf);

	strcpy(type,"--");
	if (msgStatus->isSpam)
		type[0] = 'S';
	if (msgStatus->isWhitelisted)
		type[1] = 'W';

	sprintf(name,"%s/%s",msgStatus->sessionInfo->userName,msgStatus->sessionInfo->serverName);

	switch(msgStatus->xSpamPalHeader.reason) {
		case REASON_NONE:			strcpy(spam_r,"none"); break;
		case REASON_BLIST_FROM:		strcpy(spam_r,"blist-from"); break;
		case REASON_BLIST_IP:		strcpy(spam_r,"blist-ip");
									sprintf(the_ip,"%d.%d.%d.%d",
										msgStatus->xSpamPalHeader.details.ipBlock.ipAddr.octets[0],
										msgStatus->xSpamPalHeader.details.ipBlock.ipAddr.octets[1],
										msgStatus->xSpamPalHeader.details.ipBlock.ipAddr.octets[2],
										msgStatus->xSpamPalHeader.details.ipBlock.ipAddr.octets[3]
									);
									break;
		case REASON_WLIST_FROM:		strcpy(spam_r,"wlist-from"); break;
		case REASON_WLIST_IP:		strcpy(spam_r,"wlist-ip");
									sprintf(the_ip,"%d.%d.%d.%d",
										msgStatus->xSpamPalHeader.details.ipBlock.ipAddr.octets[0],
										msgStatus->xSpamPalHeader.details.ipBlock.ipAddr.octets[1],
										msgStatus->xSpamPalHeader.details.ipBlock.ipAddr.octets[2],
										msgStatus->xSpamPalHeader.details.ipBlock.ipAddr.octets[3]
									);
									break;
		case REASON_AWLIST_FROM:	strcpy(spam_r,"awlist-from"); break;
		case REASON_AWLIST_IP:		strcpy(spam_r,"awlist-ip");
									sprintf(the_ip,"%d.%d.%d.%d",
										msgStatus->xSpamPalHeader.details.ipBlock.ipAddr.octets[0],
										msgStatus->xSpamPalHeader.details.ipBlock.ipAddr.octets[1],
										msgStatus->xSpamPalHeader.details.ipBlock.ipAddr.octets[2],
										msgStatus->xSpamPalHeader.details.ipBlock.ipAddr.octets[3]
									);
									break;
		case REASON_DNSBL:			strcpy(spam_r,"dnsbl");
									sprintf(the_ip,"%d.%d.%d.%d",
										msgStatus->xSpamPalHeader.details.ipBlock.ipAddr.octets[0],
										msgStatus->xSpamPalHeader.details.ipBlock.ipAddr.octets[1],
										msgStatus->xSpamPalHeader.details.ipBlock.ipAddr.octets[2],
										msgStatus->xSpamPalHeader.details.ipBlock.ipAddr.octets[3]
									);
									if (msgStatus->xSpamPalHeader.details.ipBlock.dnsblList)
										strcpy(dnsbl,msgStatus->xSpamPalHeader.details.ipBlock.dnsblList->name);
									break;
		case REASON_OTHER:		{
									if (strstr(msgStatus->xSpamPalHeader.details.unknown.description, "REGEX"))
										strcpy(spam_r,"regex");
									else if (strstr(msgStatus->xSpamPalHeader.details.unknown.description, "BADWORD"))
										strcpy(spam_r,"badword");
									else if (strstr(msgStatus->xSpamPalHeader.details.unknown.description, "GOODWORD"))
										strcpy(spam_r,"goodword");
									else if (strstr(msgStatus->xSpamPalHeader.details.unknown.description, "BAYESIAN"))
										strcpy(spam_r,"bayesian");
									else if (strstr(msgStatus->xSpamPalHeader.details.unknown.description, "TIME-OUT"))
										strcpy(spam_r,"timeout");
									else if (strstr(msgStatus->xSpamPalHeader.details.unknown.description, "SURBL"))
										strcpy(spam_r,"surbl");
									else if (strstr(msgStatus->xSpamPalHeader.details.unknown.description, "HTMLM"))
										strcpy(spam_r,"htmlm");
									else
										strcpy(spam_r,"other");
								}	break;
		default:					strcpy(spam_r,"unknown"); break;
	}

	if (fh!=NULL)  fprintf(fh,"\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\r\n",
									type,name,spam_r,dnsbl,the_ip,fr_buf,to_buf,su_buf);
	if (fh2!=NULL) fprintf(fh2,"\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\r\n",
									type,name,spam_r,dnsbl,the_ip,fr_buf,to_buf,su_buf);
	fclose(fh2);
	fclose(fh);
	LeaveCriticalSection(&critSect);
}







/*******************************************************************/
/************************* Help fuctions ***************************/
/*******************************************************************/

int getMaxBytesToAdd(DWORD config) {
	return 0;
}

int getAddedHeaderLines(
	unsigned long fetchHandle,
	const struct MessageStatus* msgStatus,
	DWORD config,
	char* buf,
	int bufSize
) {
	if (bufSize>=1)
		buf[0] = EOS;
	return 1;
}

int getMessageBodyPrefix(
	unsigned long fetchHandle,
	const struct MessageStatus* msgStatus,
	DWORD config,
	char* buf,
	int bufSize
) {
	if (bufSize>=1)
		buf[0] = EOS;
	return 1;
}

// Perform various tidying-up operations
void housekeeping(
	DWORD configHandle,
	BOOL *exitFlag
) {
}

/*******************************************************************/
/************************* Plugin Windows **************************/
/*******************************************************************/

BOOL checkSpampalVersion(const char* version) {
	char spampalExePath[MAX_PATH];
	int len, pos;

	len = GetModuleFileName(GetModuleHandle(NULL), spampalExePath, MAX_PATH);
	pos = len-1;
	while (pos>=0 && spampalExePath[pos]!='\\') pos--;
	pos++;

	if (stricmp(spampalExePath+pos,"spampal.exe")) {
		MessageBox(
			NULL,
			"Logfile plugin does only work with official releases of SpamPal",
			"Plugin loading failed",
			MB_OK);
		return FALSE;
	}

	if (atof(version)>=1.07) {
		return TRUE;
	}
	else {
		MessageBox(
			NULL,
			"Logfile plugin needs SpamPal v1.07 or later!",
			"Error from SpamPal Logfile plugin",
			MB_ICONWARNING | MB_SETFOREGROUND | MB_OK
		);
		return FALSE;
	}
}

BOOL hasPluginWindow() {
	return FALSE;
}

HWND createPluginWindow(DWORD config) {
	return NULL;
}

void filterOnEmailAddress(
	const char* emailAddress,
	int addressSource,
	unsigned long fetchHandle,
	struct MessageStatus* msgStatus,
	DWORD configHandle,
	const struct SpamPalFilteringHooks* spampalHooks
) {
}

void filterOnHost(
	const char* host,
	unsigned long fetchHandle,
	struct MessageStatus* msgStatus,
	DWORD configHandle,
	const struct SpamPalFilteringHooks* spampalHooks
) {
}

void filterOnIpAddress(
	const struct IpAddress* ipAddr,
	unsigned long fetchHandle,
	struct MessageStatus* msgStatus,
	DWORD configHandle,
	const struct SpamPalFilteringHooks* spampalHooks
) {
}

BOOL requireHeaderRewriting(
	const char** headerLines,
	int num_header_lines,
	unsigned long fetchHandle,
	const struct MessageStatus* msgStatus,
	DWORD configHandle
) {
	return FALSE;
}


void rewriteMessageHeaders(
	const char** headerLines,
	int num_header_lines,
	int num_header_lines_already_sent,
	HANDLE newHeadersHandle,
	unsigned long fetchHandle,
	const struct MessageStatus* msgStatus,
	DWORD configHandle
) {
}

BOOL requireBodyRewriting(
	const char** headerLines,
	int num_header_lines,
	unsigned long fetchHandle,
	const struct MessageStatus* msgStatus,
	DWORD configHandle
) {
	return FALSE;
}

void rewriteMessageBody(
	const char** headerLines,
	int num_header_lines,
	HANDLE oldBodyHandle,
	HANDLE newBodyHandle,
	unsigned long fetchHandle,
	struct MessageStatus* msgStatus,
	DWORD configHandle
) {
}
